-module(n4u_cowboy).
-behaviour(cowboy_handler).

-export([init/2, terminate/3]).

-export([params/1, form/1, path/1, request_body/1, headers/1, get_header/3,
  set_header/3, response/2, reply/2, cookies/1, cookie/2, cookie/3, cookie/5,
  delete_cookie/2, peer/1]).

-record(state, {headers, body}).


% N4U Cowboy Server Bridge


% Cowboy HTTP Handler

init(Req, _Opts) ->
  Req2 = n4u_document:run(Req),
  %wf:info(?MODULE, "init: ~p~n", [Req2]),
  {ok, Req2, #state{}}.


terminate(_Reason, _Req, _State) -> ok.


% Cowboy Bridge Abstraction

params(Req) -> cowboy_req:parse_qs(Req). % [{Key, Value}]


cookie(Cookie, Req) ->
  #{Cookie := Value} = cowboy_req:match_cookies([{Cookie, [], undefined}], Req),
  Value.


path(Req) -> cowboy_req:path(Req).


peer(Req) -> cowboy_req:peer(Req).


get_header(Name, Req, Default) -> cowboy_req:header(Name, Req, Default).


form(Req) ->
  {ok, Params, NewReq} = cowboy_req:read_urlencoded_body(Req),
  {Params, NewReq}.


request_body(Req) -> cowboy_req:read_body(Req).


headers(Req) -> cowboy_req:headers(Req). % Headers % #{headers := Headers} = Req.


set_header(Name, Value, Req) -> cowboy_req:set_resp_header(Name, Value, Req).


response(Html, Req) -> cowboy_req:set_resp_body(Html, Req).


reply(StatusCode, Req) -> cowboy_req:reply(StatusCode, Req). % Req2


cookies(Req) -> cowboy_req:parse_cookies(Req). % [{Name, Value}]


cookie(Cookie, Value, Req) -> cookie(Cookie, Value, <<"/">>, 0, Req).


cookie(Name, Value, Path, TTL, Req) ->
  Options = #{path => Path, max_age => TTL},
  cowboy_req:set_resp_cookie(Name, Value, Req, Options).


delete_cookie(Cookie, Req) -> cookie(Cookie, <<"">>, <<"/">>, 0, Req).

