-module(n4u_stream).
-behaviour(cowboy_websocket).

-include_lib("n4u/include/n4u.hrl").

-export([init/2, websocket_init/1, websocket_handle/2, websocket_info/2, terminate/3]).


% N4U Stream Bridge to WebSocket


init(R, S)                                  -> upgrade(cowboy_req:header(<<"upgrade">>, R), {R, S}).
websocket(R, <<"websocket">>, _S) ->
  %wf:info(?MODULE, "websocket: ~p~n ~p~n", [R, S]),
  {cowboy_websocket, R, R}; %{cowboy_websocket, R, S};
websocket(R, _, _S)                         -> down(reply([], R, 501)).
upgrade(undefined, {R, _S})                 -> down(reply([], R, 501));
upgrade(B, {R, S}) when erlang:is_binary(B) -> websocket(R, cowboy_bstr:to_lower(B), S).

down(R)                    -> {shutdown, R, undefined}.
reply(D, R, Code)          -> cowboy_req:reply(Code, [], D, R). % R2.


% WebSocket

websocket_init(S)      -> ws(n4u_proto:init([], S, [{formatter, bert}], ws)).
websocket_handle(D, S) ->
  %wf:info(?MODULE, "websocket_handle: ~p~n~n ~p~n~n", [S#cx.req, S]),
  ws(n4u_proto:stream(D, S#cx.req, S)).
websocket_info(D, S)   -> ws(n4u_proto:info(D, ?REQ, S)).
terminate(_M, R, S)    ->
  wf:info(?MODULE, "terminate: ~p~n", [_M]),
  n4u_proto:terminate(R, S).

ws({ok, _R, S})                    -> {ok, S};
ws({shutdown, _R, S})              -> {shutdown, S};
ws({reply, {binary, Rep}, _R, S})  -> {reply, {binary, Rep}, S};
ws({reply, Rep, _R, S})            -> {reply, {text, Rep}, S};
ws(X) ->
  wf:info(?MODULE, "ws: UNKNOWN: ~p~n", [X]),
  {shutdown, []}.

